@@ -14,7 +14,6 @@
 #include <kopano/ECTags.h>
 #include <kopano/IECInterfaces.hpp>
 #include <kopano/automapi.hpp>
-#include <kopano/ecversion.h>
 #include <kopano/memory.hpp>
 #include <kopano/ustringutil.h>
 
@@ -54,7 +53,9 @@ extern _kc_export HRESULT HrCreateEmailSearchKey(const char *type, const char *a
 extern _kc_export HRESULT DoSentMail(IMAPISession *, IMsgStore *, ULONG flags, object_ptr<IMessage>);
 extern _kc_export HRESULT GetClientVersion(unsigned int *);
 extern _kc_export HRESULT OpenSubFolder(LPMDB, const wchar_t *folder, wchar_t psep, bool is_public, bool create_folder, LPMAPIFOLDER *subfolder);
+extern _kc_export HRESULT spv_postload_large_props(IMAPIProp *, const SPropTagArray *, unsigned int, SPropValue *);
 extern _kc_export HRESULT HrOpenDefaultCalendar(LPMDB, LPMAPIFOLDER *default_folder);
+extern _kc_export HRESULT HrGetFullProp(IMAPIProp *prop, unsigned int tag, SPropValue **);
 extern _kc_export HRESULT HrGetAllProps(IMAPIProp *prop, ULONG flags, ULONG *nvals, LPSPropValue *props);
 extern _kc_export HRESULT UnWrapStoreEntryID(ULONG eid_size, const ENTRYID *eid, ULONG *ret_size, ENTRYID **ret);
 extern _kc_export HRESULT GetECObject(IMAPIProp *, const IID &, void **);
@@ -131,7 +132,7 @@ class _kc_export KServerContext {
 	HRESULT logon(const char *user = nullptr, const char *password = nullptr);
 	HRESULT inbox(IMAPIFolder **) const;
 
-	const char *m_app_misc = nullptr, *m_app_ver = PROJECT_VERSION, *m_host = nullptr;
+	const char *m_app_misc = nullptr, *m_host = nullptr;
 	const char *m_ssl_keyfile = nullptr, *m_ssl_keypass = nullptr;
 	unsigned int m_ses_flags = EC_PROFILE_FLAGS_NO_NOTIFICATIONS;
 
