@@ -7,17 +7,15 @@
 #define ECLOGGER_H
 
 #include <atomic>
-#include <kopano/zcdefs.h>
-#include <kopano/platform.h>
+#include <clocale>
+#include <cstdarg>
 #include <list>
 #include <memory>
 #include <mutex>
-#include <pthread.h>
-#include <csignal>
-#include <cstdarg>
-#include <cstdio>
 #include <string>
-#include <kopano/memory.hpp>
+#include <sys/types.h>
+#include <kopano/zcdefs.h>
+#include <kopano/platform.h>
 #ifndef KC_LIKE_PRINTF
 #	define KC_LIKE_PRINTF(_fmt, _va)
 #endif
@@ -25,7 +23,6 @@
 namespace KC {
 
 class ECConfig;
-class ECLogger;
 
 static const unsigned int EC_LOGLEVEL_NONE       = 0;
 static const unsigned int EC_LOGLEVEL_FATAL	 = 1;
@@ -227,23 +224,6 @@ class KC_EXPORT_DYCAST ECLogger_File KC_FINAL : public ECLogger {
 	_kc_hidden void init_for_gzfile(void);
 };
 
-/**
- * Linux syslog logger. Output is whatever syslog does, probably LC_CTYPE.
- */
-class KC_EXPORT_DYCAST ECLogger_Syslog KC_FINAL : public ECLogger {
-	private:
-	std::unique_ptr<char[], cstdlib_deleter> m_ident;
-	static const int levelmap[16]; /* converts to syslog levels */
-
-	public:
-	ECLogger_Syslog(unsigned int max_ll, const char *ident, int facility);
-	~ECLogger_Syslog(void);
-	_kc_hidden virtual void Reset(void) _kc_override;
-	_kc_hidden virtual void log(unsigned int level, const char *msg) _kc_override;
-	_kc_hidden virtual void logf(unsigned int level, const char *fmt, ...) _kc_override KC_LIKE_PRINTF(3, 4);
-	_kc_hidden virtual void logv(unsigned int level, const char *fmt, va_list &) _kc_override;
-};
-
 /**
  * Pipe Logger, only used by forked model processes. Redirects every
  * log message to an ECLogger_File object. This ECLogger_Pipe object
@@ -309,7 +289,8 @@ extern _kc_export void ec_log(unsigned int level, const std::string &msg);
 extern _kc_export HRESULT ec_log_hrcode(HRESULT, unsigned int level, const char *fmt, const char *func);
 extern _kc_export ECLogger *CreateLogger(ECConfig *, const char *argv0, const char *service, bool audit = false);
 extern _kc_export void LogConfigErrors(ECConfig *);
-extern _kc_export void generic_sigsegv_handler(ECLogger *, const char *app, const char *vers, int sig, const siginfo_t *, const void *uctx);
+extern _kc_export void ec_setup_segv_handler(const char *app, const char *vers);
+extern _kc_export const std::string &ec_os_pretty_name();
 
 } /* namespace */
 
