@@ -7,13 +7,13 @@
 
 #include <string>
 #include <vector>
-#include <kopano/zcdefs.h>
 #include <dirent.h>
-#include <sys/resource.h>
-#include <kopano/ECConfig.h>
+#include <kopano/zcdefs.h>
 
 namespace KC {
 
+class ECConfig;
+
 class fs_deleter {
 	public:
 	void operator()(DIR *d) const
@@ -27,10 +27,10 @@ extern _kc_export int unix_runas(ECConfig *);
 extern _kc_export int unix_chown(const char *filename, const char *user, const char *group);
 extern _kc_export void unix_coredump_enable(const char *);
 extern _kc_export int unix_create_pidfile(const char *argv0, ECConfig *, bool force = true);
-extern _kc_export int unix_daemonize(ECConfig *);
 extern _kc_export int unix_fork_function(void *(*)(void *), void *param, int nfds, int *closefds);
 extern _kc_export bool unix_system(const char *logname, const std::vector<std::string> &cmd, const char **env);
 extern _kc_export int ec_reexec(const char *const *);
+extern _kc_export void ec_reexec_finalize();
 
 } /* namespace */
 
