@@ -189,10 +189,7 @@ public:
 	 * Convert the entryid to a human readable hexadecimal format.
 	 * @return The entryid in hexadecimal format.
 	 */
-	_kc_hidden std::string tostring(void) const
-	{
-		return bin2hex(m_eid.size(), m_eid.data());
-	}
+	_kc_hidden std::string tostring() const { return bin2hex(m_eid); }
 
 	/**
 	 * Get entryid as a std::string
@@ -392,9 +389,7 @@ public:
 	 * Convert the entryid to a human readable hexadecimal format.
 	 * @return The entryid in hexadecimal format.
 	 */
-	std::string tostring() const {
-		return bin2hex(m_eid.size(), m_eid.data());
-	}
+	std::string tostring() const { return bin2hex(m_eid); }
 
 private:
 	/**
@@ -444,8 +439,8 @@ static inline bool operator!=(const SObjectEntry &lhs, const SObjectEntry &rhs)
 }
 
 static inline bool operator<(const SObjectEntry &lhs, const SObjectEntry &rhs) {
-	return lhs.sStoreEntryId < rhs.sStoreEntryId ||
-			(lhs.sStoreEntryId == rhs.sStoreEntryId && lhs.sItemEntryId < rhs.sItemEntryId);
+	return std::tie(lhs.sStoreEntryId, lhs.sItemEntryId) <
+	       std::tie(rhs.sStoreEntryId, rhs.sItemEntryId);
 }
 
 
