@@ -75,26 +75,20 @@ typedef signed char		CHAR;
 typedef BYTE*			LPBYTE;
 typedef unsigned short	WORD;
 typedef WORD*			LPWORD;
-typedef unsigned short	USHORT;
-typedef USHORT*			LPUSHORT;
 typedef unsigned int	DWORD;
 typedef DWORD*			LPDWORD;
 typedef unsigned int	UINT;
-typedef UINT*			LPUINT;
 typedef unsigned int	ULONG;
 typedef ULONG*			LPULONG;
 typedef int64_t		LONGLONG;
 typedef LONGLONG*		LPLONGLONG;
 typedef uint64_t	ULONGLONG;
-typedef ULONGLONG*		LPULONGLONG;
 typedef uintptr_t ULONG_PTR;
 typedef int	LONG;
 typedef int	BOOL;
 typedef void* LPVOID;
-typedef const void* LPCVOID;
 typedef char* LPSTR;
 typedef const char* LPCSTR;
-typedef void* HGLOBAL;
 typedef __int64_t __int64;
 
 #ifndef FALSE
@@ -130,10 +124,8 @@ typedef CLSID* LPCLSID;
 typedef GUID   FMTID;
 typedef FMTID* LPFMTID;
 
-#define REFGUID  const GUID &
 #define REFIID   const IID &
 #define REFCLSID const CLSID &
-#define REFFMTID const FMTID &
 
 typedef GUID  UUID;		// needed? existing?
 
@@ -195,7 +187,6 @@ typedef GUID  UUID;		// needed? existing?
 /* correct? */
 typedef unsigned int	HINSTANCE;
 typedef unsigned int	HANDLE;
-typedef unsigned int	HWND;
 typedef int				HRESULT;
 typedef int				SCODE;
 
@@ -234,7 +225,7 @@ typedef int				SCODE;
 #define STG_E_INVALIDFUNCTION KC_HRESULT_TYPEDEF_(0x80030001L)
 
 /*
- Values are 32 bit values layed out as follows:
+ Values are 32 bit values laid out as follows:
 
   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
@@ -282,7 +273,6 @@ typedef WCHAR		TCHAR;
 namespace KC {
 typedef std::basic_string<TCHAR> tstring;
 }
-typedef unsigned char	TBYTE;
 typedef WCHAR*			LPWSTR;
 typedef const WCHAR*	LPCWSTR;
 typedef TCHAR*			LPTSTR;
@@ -400,23 +390,6 @@ extern "C" {
 extern _kc_export HRESULT CoCreateGuid(LPGUID);
 extern _kc_export void GetSystemTimeAsFileTime(FILETIME *);
 
-/* Some wrappers to map Windows unicode functions */
-static inline int lstrcmpW(LPCWSTR str1, LPCWSTR str2)
-{
-	return wcscmp((WCHAR *)str1, (WCHAR *)str2);
-}
-
-static inline int lstrlenW(LPCWSTR str)
-{
-	return wcslen((WCHAR *)str);
-}
-
-static inline LPWSTR lstrcpyW(LPWSTR dst, LPCWSTR src)
-{
-	wcscpy(dst, (WCHAR *)src);
-	return dst;
-}
-
 #define _tcslen	wcslen
 #define _tcscpy wcscpy
 #define _tcscmp wcscmp
@@ -426,7 +399,7 @@ extern _kc_export void Sleep(unsigned int usec);
 
 /* because the flags are not used linux, they do not match the windows flags! */
 #define GPTR 0
-extern _kc_export HGLOBAL GlobalAlloc(UINT flags, ULONG size);
+extern _kc_export void * GlobalAlloc(UINT flags, ULONG size);
 
 } /* extern "C" */
 
@@ -450,7 +423,7 @@ extern _kc_export time_t GetProcessTime(void);
 #define TICKS_PER_SEC (sysconf(_SC_CLK_TCK))
 
 /**
- * Tell the compiler that a funtion/method behaves like printf.
+ * Tell the compiler that a function/method behaves like printf.
  * _fmt is the 1-indexed index to the format argument
  * _va is the 1-indexed index to the first va argument.
  * NOTE: For non-static methods the this pointer has index 1.
