@@ -5,17 +5,19 @@
 #ifndef _STRINGUTIL_H
 #define _STRINGUTIL_H
 
-#include <kopano/zcdefs.h>
-#include <cstdarg>
+#include <algorithm>
+#include <cctype>
+#include <cstdint>
+#include <cstdlib>
+#include <cstring>
+#include <cwctype>
 #include <iterator>
 #include <set>
 #include <string>
 #include <type_traits>
 #include <vector>
-#include <algorithm>
-#include <cctype>
-#include <cwctype>
 #include <kopano/platform.h>
+#include <kopano/zcdefs.h>
 #include <openssl/md5.h>
 
 struct SBinary;
@@ -41,12 +43,12 @@ struct wcscasecmp_comparison {
 };
 
 static inline std::string strToUpper(std::string f) {
-	transform(f.begin(), f.end(), f.begin(), ::toupper);
+	std::transform(f.begin(), f.end(), f.begin(), ::toupper);
 	return f;
 }
 
 static inline std::string strToLower(std::string f) {
-	transform(f.begin(), f.end(), f.begin(), ::tolower);
+	std::transform(f.begin(), f.end(), f.begin(), ::tolower);
 	return f;
 }
 
@@ -58,7 +60,7 @@ static inline std::wstring strToUpper(std::wstring f)
 
 static inline std::string stringify(unsigned int x)
 {
-	/* (w)stringify(-1) has a different result than to_(w)string(-1), so do not subtitute! */
+	/* (w)stringify(-1) has a different result than to_(w)string(-1), so do not substitute! */
 	return std::to_string(x);
 }
 
@@ -101,6 +103,7 @@ extern _kc_export std::string bin2hex(size_t len, const void *input);
 extern _kc_export std::string bin2hex(const SBinary &);
 extern _kc_export std::string bin2txt(const void *, size_t);
 extern _kc_export std::string bin2txt(const SBinary &);
+inline std::string bin2txt(const std::string &s) { return bin2txt(s.data(), s.size()); }
 extern _kc_export std::string urlEncode(const std::string &);
 extern _kc_export std::string urlEncode(const std::wstring &, const char *charset);
 extern _kc_export std::string urlEncode(const wchar_t *input, const char *charset);
@@ -215,7 +218,6 @@ kc_join(const C &v, const char *sep, F &&func)
 extern _kc_export std::string base64_encode(const void *, unsigned int);
 extern _kc_export std::string base64_decode(const std::string &);
 extern _kc_export std::string zcp_md5_final_hex(MD5_CTX *);
-extern _kc_export std::wstring string_strip_nuls(const std::wstring &);
 extern _kc_export std::string string_strip_crlf(const char *);
 extern _kc_export bool SymmetricIsCrypted(const char *);
 extern _kc_export std::string SymmetricDecrypt(const char *);
