@@ -3,7 +3,7 @@
 Part of the high-level python bindings for Kopano.
 
 Copyright 2005 - 2016 Zarafa and its licensors (see LICENSE file)
-Copyright 2016 - Kopano and its licensors (see LICENSE file)
+Copyright 2016 - 2019 Kopano and its licensors (see LICENSE file)
 """
 
 import sys
@@ -32,55 +32,57 @@ from .compat import (
     benc as _benc, bdec as _bdec, fake_unicode as _unicode,
 )
 
-if sys.hexversion >= 0x03000000:
-    try:
-        from . import server as _server
-    except ImportError: # pragma: no cover
-        _server = sys.modules[__package__ + '.server']
-    try:
-        from . import store as _store
-    except ImportError: # pragma: no cover
-        _store = sys.modules[__package__ + '.store']
-else: # pragma: no cover
-    import server as _server
-    import store as _store
+try:
+    from . import server as _server
+except ImportError: # pragma: no cover
+    _server = sys.modules[__package__ + '.server']
+try:
+    from . import store as _store
+except ImportError: # pragma: no cover
+    _store = sys.modules[__package__ + '.store']
 
 class Company(Properties):
-    """Company class"""
+    """Company class
+
+    For a multi-tenant Kopano setup, users are grouped in companies.
+    Regular users are unaware of anything outside their own company.
+    """
 
     def __init__(self, name, server=None):
-        self.server = server or _server.Server()
+        self.server = server or \
+            _server.Server(_skip_check=True, parse_args=False)
 
         self._name = name = _unicode(name)
-        if name != u'Default': # XXX
+        if name != 'Default': # TODO
             try:
                 id_ = self.server.sa.ResolveCompanyName(name, MAPI_UNICODE)
                 self._eccompany = self.server.sa.GetCompany(id_, MAPI_UNICODE)
             except MAPIErrorNotFound:
                 raise NotFoundError("no such company: '%s'" % name)
 
-        # XXX cached because GetPublicStore does not see changes..
+        # TODO cached because GetPublicStore does not see changes..
         #     do we need folder & store notifications (slow)?
         self._public_store = None
-
         self._mapiobj = None
 
     @property
     def mapiobj(self):
+        """Underlying MAPI object."""
         if not self._mapiobj:
             id_ = self._eccompany.CompanyID
             self._mapiobj = self.server.mapisession.OpenEntry(id_, None, 0)
         return self._mapiobj
 
     @property
-    def companyid(self): # XXX single-tenant case
-        if self._name != u'Default':
+    def companyid(self): # TODO single-tenant case
+        """Company id."""
+        if self._name != 'Default':
             return _benc(self._eccompany.CompanyID)
 
     @property
     def admin(self):
         """Company :class:`administrator <User>` in multi-tenant mode."""
-        if self._name != u'Default':
+        if self._name != 'Default':
             id_ = self._eccompany.AdministratorID
             ecuser = self.server.sa.GetUser(id_, MAPI_UNICODE)
             return self.server.user(ecuser.Username)
@@ -93,7 +95,7 @@ class Company(Properties):
     @property
     def hidden(self):
         """The company is hidden from the addressbook."""
-        if self._name != u'Default':
+        if self._name != 'Default':
             return bool(self._eccompany.IsHidden)
         return False
 
@@ -110,10 +112,9 @@ class Company(Properties):
         self.server.sa.SetCompany(self._eccompany, MAPI_UNICODE)
 
     def store(self, guid):
-        """Store for the given GUID
+        """:class:`store <Store>` for the given GUID
 
         :param guid: store guid
-        :return: :class:`store <Store>` with given GUID.
         """
         if guid == 'public':
             if not self.public_store:
@@ -129,7 +130,9 @@ class Company(Properties):
         if self.server.multitenant:
             table = self.server.sa.OpenUserStoresTable(MAPI_UNICODE)
             restriction = SPropertyRestriction(
-                RELOP_EQ, PR_EC_COMPANY_NAME_W, SPropValue(PR_EC_COMPANY_NAME_W, self.name)
+                RELOP_EQ,
+                PR_EC_COMPANY_NAME_W,
+                SPropValue(PR_EC_COMPANY_NAME_W, self.name)
             )
             table.Restrict(restriction, TBL_BATCH)
             for row in table.QueryRows(-1, 0):
@@ -147,70 +150,84 @@ class Company(Properties):
     def public_store(self):
         """Company public :class:`store <Store>`."""
         if not self._public_store:
-            if self._name == u'Default': # XXX
+            if self._name == 'Default': # TODO
                 pubstore = GetPublicStore(self.server.mapisession)
                 if pubstore is None:
                     self._public_store = None
                 else:
-                    self._public_store = _store.Store(mapiobj=pubstore, server=self.server)
+                    self._public_store = \
+                        _store.Store(mapiobj=pubstore, server=self.server)
             else:
                 try:
-                    entryid = self.server.ems.CreateStoreEntryID(None, self._name, MAPI_UNICODE)
+                    entryid = self.server.ems.CreateStoreEntryID(None,
+                        self._name, MAPI_UNICODE)
                 except MAPIErrorNotFound:
                     self._public_store = None
                 else:
-                    self._public_store = _store.Store(entryid=_benc(entryid), server=self.server)
+                    self._public_store = \
+                        _store.Store(entryid=_benc(entryid),server=self.server)
 
         return self._public_store
 
     def create_public_store(self):
         """Create company public :class:`store <Store>`."""
-        if self._name == u'Default':
+        if self._name == 'Default':
             try:
-                storeid_rootid = self.server.sa.CreateStore(ECSTORE_TYPE_PUBLIC, EID_EVERYONE)
+                storeid_rootid = self.server.sa.CreateStore(
+                    ECSTORE_TYPE_PUBLIC, EID_EVERYONE)
             except MAPIErrorCollision:
                 raise DuplicateError("public store already exists")
         else:
             try:
-                storeid_rootid = self.server.sa.CreateStore(ECSTORE_TYPE_PUBLIC, self._eccompany.CompanyID)
+                storeid_rootid = self.server.sa.CreateStore(
+                    ECSTORE_TYPE_PUBLIC, self._eccompany.CompanyID)
             except MAPIErrorCollision:
-                raise DuplicateError("public store already exists for company '%s'" % self.name)
+                raise DuplicateError(
+                    "public store already exists for company '%s'" % self.name)
 
-        store_entryid = WrapStoreEntryID(0, b'zarafa6client.dll', storeid_rootid[0][:-4]) + self.server.pseudo_url + b'\x00'
+        store_entryid = WrapStoreEntryID(0, b'zarafa6client.dll',
+            storeid_rootid[0][:-4]) + self.server.pseudo_url + b'\x00'
 
-        self._public_store = _store.Store(entryid=_benc(store_entryid), server=self.server)
+        self._public_store = _store.Store(entryid=_benc(store_entryid),
+            server=self.server)
         return self._public_store
 
     def hook_public_store(self, store):
         """Hook company public :class:`store <Store>`.
 
-        :param store: store to hook as public store
+        :param store: :class:`store <Store>` to hook as public store
         """
-        if self._name == u'Default':
+        if self._name == 'Default':
             try:
-                self.server.sa.HookStore(ECSTORE_TYPE_PUBLIC, EID_EVERYONE, _bdec(store.guid))
+                self.server.sa.HookStore(ECSTORE_TYPE_PUBLIC, EID_EVERYONE,
+                    _bdec(store.guid))
             except MAPIErrorCollision:
                 raise DuplicateError("hooked public store already exists")
         else:
             try:
-                self.server.sa.HookStore(ECSTORE_TYPE_PUBLIC, _bdec(self.companyid), _bdec(store.guid))
+                self.server.sa.HookStore(ECSTORE_TYPE_PUBLIC,
+                    _bdec(self.companyid), _bdec(store.guid))
             except MAPIErrorCollision:
-                raise DuplicateError("hooked public store already exists for company '%s'" % self.name)
+                raise DuplicateError(
+                    "hooked public store already exists for company '%s'" %
+                    self.name)
 
         self._public_store = store
 
     def unhook_public_store(self):
         """Unhook company public :class:`store <Store>`."""
-        if self._name == u'Default':
+        if self._name == 'Default':
             try:
                 self.server.sa.UnhookStore(ECSTORE_TYPE_PUBLIC, EID_EVERYONE)
             except MAPIErrorNotFound:
                 raise NotFoundError("no hooked public store")
         else:
             try:
-                self.server.sa.UnhookStore(ECSTORE_TYPE_PUBLIC, _bdec(self.companyid))
+                self.server.sa.UnhookStore(ECSTORE_TYPE_PUBLIC,
+                    _bdec(self.companyid))
             except MAPIErrorNotFound:
-                raise NotFoundError("no hooked public store for company '%s'" % self.name)
+                raise NotFoundError(
+                    "no hooked public store for company '%s'" % self.name)
         self._public_store = None
 
     def user(self, name, create=False):
@@ -218,6 +235,8 @@ class Company(Properties):
 
         :param name: user name
         :param create: create user if it doesn't exist (default False)
+
+        :raises: NotFoundError
         """
         if '@' not in name and self._name != 'Default':
             name = name + '@' + self._name
@@ -241,10 +260,10 @@ class Company(Properties):
 
     def users(self, **kwargs):
         """
+        Return all :class:`users <User>` within company.
 
         :param parse: filter users on cli argument --user (default True)
         :param system: include system users (default False)
-        :return: all :class:`users <User>` within company
         """
 
         if 'remote' not in kwargs:
@@ -258,21 +277,26 @@ class Company(Properties):
 
         :param name: user name
         :param password: password (default None)
+
+        :raises: DuplicateError
         """
         name = name.split('@')[0]
-        if self._name == u'Default':
+        if self._name == 'Default':
             self.server.create_user(name, password=password)
         else:
-            self.server.create_user(name, password=password, company=self._name)
+            self.server.create_user(name, password=password,
+                company=self._name)
         return self.user(name)
 
     def group(self, name, create=False):
         """Return :class:`group <Group>` with given name.
 
         :param name: group name
+        :param create: create group if it doesn't exist (default False)
+
         :raises: NotFoundError
         """
-        for group in self.groups(): # XXX
+        for group in self.groups(): # TODO
             if group.name == name:
                 return group
         if create:
@@ -281,7 +305,8 @@ class Company(Properties):
             raise NotFoundError("no such group: '%s'" % name)
 
     def get_group(self, name):
-        """Return :class:`group <Group>` with given name or *None* if not found.
+        """Return :class:`group <Group>` with given name or *None*
+        if not found.
 
         :param name: group name
         """
@@ -292,23 +317,31 @@ class Company(Properties):
 
     def groups(self):
         """Return all :class:`groups <Group>` within the company."""
-        if self.name == u'Default': # XXX
+        if self.name == 'Default': # TODO
             for group in self.server.groups():
                 yield group
         else:
-            for ecgroup in self.server.sa.GetGroupList(self._eccompany.CompanyID, MAPI_UNICODE):
+            for ecgroup in self.server.sa.GetGroupList(
+                    self._eccompany.CompanyID, MAPI_UNICODE):
                 yield Group(ecgroup.Groupname, self.server)
 
     def create_group(self, name):
+        """Create :class:`group <Group>`.
+
+        :param name: group name
+        :raises: DuplicateError
+        """
         name = name.split('@')[0]
-        if self._name == u'Default':
+        if self._name == 'Default':
             self.server.create_group(name)
         else:
             self.server.create_group(name+'@'+self._name)
         return self.group(name)
 
     def admins(self):
-        for ecuser in self.server.sa.GetRemoteAdminList(self._eccompany.CompanyID, MAPI_UNICODE):
+        """Return all company :class:`admins <User>`."""
+        for ecuser in self.server.sa.GetRemoteAdminList(
+                self._eccompany.CompanyID, MAPI_UNICODE):
             yield self.server.user(ecuser.Username)
 
     def add_admin(self, user):
@@ -318,9 +351,12 @@ class Company(Properties):
         :raises: DuplicateError
         """
         try:
-            self.server.sa.AddUserToRemoteAdminList(user._ecuser.UserID, self._eccompany.CompanyID)
+            self.server.sa.AddUserToRemoteAdminList(user._ecuser.UserID,
+                self._eccompany.CompanyID)
         except MAPIErrorCollision:
-            raise DuplicateError("user '%s' already admin for company '%s'" % (user.name, self.name))
+            raise DuplicateError(
+                "user '%s' already admin for company '%s'" %
+                (user.name, self.name))
 
     def remove_admin(self, user):
         """Remove user from admin list
@@ -329,30 +365,43 @@ class Company(Properties):
         :raises: NotFoundError
         """
         try:
-            self.server.sa.DelUserFromRemoteAdminList(user._ecuser.UserID, self._eccompany.CompanyID)
+            self.server.sa.DelUserFromRemoteAdminList(user._ecuser.UserID,
+                self._eccompany.CompanyID)
         except MAPIErrorNotFound:
-            raise NotFoundError("user '%s' no admin for company '%s'" % (user.name, self.name))
+            raise NotFoundError("user '%s' no admin for company '%s'" %
+                (user.name, self.name))
 
     def views(self):
-        for eccompany in self.server.sa.GetRemoteViewList(self._eccompany.CompanyID, MAPI_UNICODE):
+        """Return all :class:`remote viewers <Company>` (other companies
+        to which the address book is visible)."""
+        for eccompany in self.server.sa.GetRemoteViewList(
+                self._eccompany.CompanyID, MAPI_UNICODE):
             yield self.server.company(eccompany.Companyname)
 
     def add_view(self, company):
+        """Add :class:`remote viewer <Company>`."""
         try:
-            self.server.sa.AddCompanyToRemoteViewList(company._eccompany.CompanyID, self._eccompany.CompanyID)
+            self.server.sa.AddCompanyToRemoteViewList(
+                company._eccompany.CompanyID, self._eccompany.CompanyID)
         except MAPIErrorCollision:
-            raise DuplicateError("company '%s' already in view-list for company '%s'" % (company.name, self.name))
+            raise DuplicateError(
+                "company '%s' already in view-list for company '%s'" %
+                (company.name, self.name))
 
     def remove_view(self, company):
+        """Remove :class:`remote viewer <Company>`."""
         try:
-            self.server.sa.DelCompanyFromRemoteViewList(company._eccompany.CompanyID, self._eccompany.CompanyID)
+            self.server.sa.DelCompanyFromRemoteViewList(
+                company._eccompany.CompanyID, self._eccompany.CompanyID)
         except MAPIErrorNotFound:
-            raise NotFoundError("company '%s' not in view-list for company '%s'" % (company.name, self.name))
+            raise NotFoundError(
+                "company '%s' not in view-list for company '%s'" %
+                (company.name, self.name))
 
     @property
     def quota(self):
         """Company :class:`Quota`."""
-        if self._name == u'Default':
+        if self._name == 'Default':
             return Quota(self.server, None)
         else:
             return Quota(self.server, self._eccompany.CompanyID)
