@@ -3,14 +3,13 @@
 Part of the high-level python bindings for Kopano
 
 Copyright 2005 - 2016 Zarafa and its licensors (see LICENSE file)
-Copyright 2016 - Kopano and its licensors (see LICENSE file)
+Copyright 2016 - 2019 Kopano and its licensors (see LICENSE file)
 """
 
 from datetime import datetime
 import optparse
 import sys
 
-from .compat import decode as _decode
 
 def parse_date(option, opt_str, value, parser):
     setattr(parser.values, option.dest, datetime.strptime(value, '%Y-%m-%d'))
@@ -19,14 +18,16 @@ def parse_loglevel(option, opt_str, value, parser):
     setattr(parser.values, option.dest, value.upper())
 
 def parse_str(option, opt_str, value, parser):
-    setattr(parser.values, option.dest, _decode(value))
+    setattr(parser.values, option.dest, value)
 
 def parse_list_str(option, opt_str, value, parser):
-    getattr(parser.values, option.dest).append(_decode(value))
+    getattr(parser.values, option.dest).append(value)
 
 def parse_bool(option, opt_str, value, parser):
     if value.lower() not in ('yes', 'no', 'true', 'false'):
-        raise Exception("error: %s option requires 'yes', 'no', 'true' or 'false'" % opt_str)
+        raise Exception(
+            "error: %s option requires 'yes', 'no', 'true' or 'false'" %
+                opt_str)
     setattr(parser.values, option.dest, value.lower() in ('yes', 'true'))
 
 def show_version(*args):
@@ -67,7 +68,7 @@ def _date():
 def _list_name():
     return _callback(callback=parse_list_str, default=[], metavar='NAME')
 
-def parser(options='cskpUPufmvCGSlbe', usage=None):
+def parser(options='CSKQUPufmvcgslbe', usage=None):
     """
 Return OptionParser instance from the standard ``optparse`` module,
 containing common kopano command-line options
@@ -76,26 +77,28 @@ containing common kopano command-line options
 
 Available options:
 
--c, --config: Path to configuration file
+-C, --config: Path to configuration file
 
--s, --server-socket: Storage server socket address
+-S, --server-socket: Storage server socket address
 
--k, --sslkey-file: SSL key file
+-K, --sslkey-file: SSL key file
 
--p, --sslkey-password: SSL key password
+-Q, --sslkey-password: SSL key passphrase
 
 -U, --auth-user: Login as user
 
 -P, --auth-pass: Login with password
 
--C, --company: Run program for specific company
+-c, --company: Run program for specific company
 
 -u, --user: Run program for specific user
 
--S, --store: Run program for specific store
+-s, --store: Run program for specific store
 
 -f, --folder: Run program for specific folder
 
+-g, --group: Run program for specific group
+
 -b, --period-begin: Run program for specific period
 
 -e, --period-end: Run program for specific period
@@ -114,7 +117,6 @@ Available options:
 
 -V, --version: Show program version
 """
-
     parser = optparse.OptionParser(
         formatter=optparse.IndentedHelpFormatter(max_help_position=42),
         usage=usage
@@ -124,28 +126,28 @@ Available options:
     kw_date = _callback(callback=parse_date)
     kw_list_str = _callback(callback=parse_list_str)
 
-    if 'c' in options:
+    if 'C' in options:
         parser.add_option(
-            '-c', '--config', dest='config_file',
+            '-C', '--config', dest='config_file',
             help='load settings from FILE', metavar='FILE', **kw_str
         )
 
-    if 's' in options:
+    if 'S' in options:
         parser.add_option(
-            '-s', '--server-socket', dest='server_socket',
+            '-S', '--server-socket', dest='server_socket',
             help='connect to server SOCKET', metavar='SOCKET', **kw_str
         )
 
-    if 'k' in options:
+    if 'K' in options:
         parser.add_option(
-            '-k', '--ssl-key', dest='sslkey_file',
+            '-K', '--ssl-key', dest='sslkey_file',
             help='SSL key file', metavar='FILE', **kw_str
         )
 
-    if 'p' in options:
+    if 'Q' in options:
         parser.add_option(
-            '-p', '--ssl-pass', dest='sslkey_pass',
-            help='SSL key password', metavar='PASS', **kw_str
+            '-Q', '--ssl-pass', dest='sslkey_pass',
+            help='SSL key passphrase', metavar='PASS', **kw_str
         )
 
     if 'U' in options:
@@ -160,15 +162,15 @@ Available options:
             help='login with password', metavar='PASS', **kw_str
         )
 
-    if 'G' in options:
+    if 'g' in options:
         parser.add_option(
-            '-G', '--group', dest='groups', default=[],
+            '-g', '--group', dest='groups', default=[],
             help='Specify group', metavar='NAME', **kw_list_str
         )
 
-    if 'C' in options:
+    if 'c' in options:
         parser.add_option(
-            '-C', '--company', dest='companies', default=[],
+            '-c', '--company', dest='companies', default=[],
             help='Specify company', metavar='NAME', **kw_list_str
         )
 
@@ -178,9 +180,9 @@ Available options:
             help='Specify user', metavar='NAME', **kw_list_str
         )
 
-    if 'S' in options:
+    if 's' in options:
         parser.add_option(
-            '-S', '--store', dest='stores', default=[],
+            '-s', '--store', dest='stores', default=[],
             help='Specify store', metavar='GUID', **kw_list_str
         )
 
