@@ -9,12 +9,11 @@ import sys
 import traceback
 from optparse import OptionGroup
 
-from MAPI.Tags import PR_EC_STATSTABLE_SYSTEM, PR_DISPLAY_NAME, PR_EC_STATS_SYSTEM_VALUE
 import kopano
 from kopano.parser import _true, _int, _name, _guid, _bool, _list_name, _date, _path
 
 def parser_opt_args():
-    parser = kopano.parser('skpcuGCfVUPS')
+    parser = kopano.parser('SKQCugcfVUPs')
     parser.add_option('--debug', help='Debug mode', **_true())
     parser.add_option('--lang', help='Create folders in this language')
     parser.add_option('--create', help='Create object', **_true())
@@ -24,6 +23,7 @@ def parser_opt_args():
     group.add_option('--list-users', help='List users', **_true())
     group.add_option('--list-groups', help='List groups', **_true())
     group.add_option('--list-companies', help='List companies', **_true())
+    group.add_option('--list-stores', help='List stores', **_true())
     group.add_option('--list-orphans', help='List orphan stores', **_true())
     group.add_option('--user-count', help='Output user counts', **_true())
     parser.add_option_group(group)
@@ -112,9 +112,9 @@ def parser_opt_args():
 ACTION_MATRIX = {
     ('list_users',): ('global', 'companies', 'groups'),
     ('list_groups',): ('global', 'companies',),
-    ('create', 'delete'): ('companies', 'groups', 'users'),
+    ('create', 'delete'): ('companies', 'groups', 'users', 'stores'),
     ('sync', 'clear_cache', 'purge_softdelete', 'purge_deferred', 'remove_store'): ('global',),
-    ('list_orphans', 'list_companies', 'user_count'): ('global',),
+    ('list_orphans', 'list_companies', 'list_stores', 'user_count'): ('global',),
 }
 
 UPDATE_MATRIX = {
@@ -134,13 +134,6 @@ UPDATE_MATRIX = {
     ('admin', 'add_view', 'remove_view', 'add_admin', 'remove_admin'): ('companies',),
 }
 
-if sys.hexversion >= 0x03000000:
-    def _encode(s):
-        return s
-else: # pragma: no cover
-    def _encode(s):
-        return s.encode(sys.stdout.encoding or 'utf8')
-
 def orig_option(o):
     OBJ_OPT = {'users': '--user', 'groups': '--group', 'companies': '--company', 'stores': '--store'}
     return OBJ_OPT.get(o) or '--'+o.replace('_', '-')
@@ -164,8 +157,8 @@ def list_users(intro, users):
     print(fmt.format('User', 'Full Name', 'Homeserver'))
     print(58*'-')
     for user in sorted(users, key=lambda u: u.name):
-        print(fmt.format(_encode(user.name), _encode(user.fullname), _encode(user.home_server)))
-    print
+        print(fmt.format(user.name, user.fullname, user.home_server))
+    print()
 
 def list_groups(intro, groups):
     groups = list(groups)
@@ -174,8 +167,8 @@ def list_groups(intro, groups):
     print(fmt.format('Groupname'))
     print(16*'-')
     for group in sorted(groups, key=lambda g: g.name):
-        print(fmt.format(_encode(group.name)))
-    print
+        print(fmt.format(group.name))
+    print()
 
 def list_companies(intro, companies):
     companies = list(companies)
@@ -184,7 +177,14 @@ def list_companies(intro, companies):
     print(fmt.format('Companyname', 'System Administrator'))
     print(65*'-')
     for company in sorted(companies, key=lambda c: c.name):
-        print(fmt.format(_encode(company.name), _encode(company.admin.name) if company.admin else ''))
+        print(fmt.format(company.name, company.admin.name if company.admin else ''))
+
+def list_stores(server):
+    fmt = '{0:<32}'
+    print(fmt.format('Store guid'))
+    print(32*'-')
+    for store in server.stores():
+        print(fmt.format(store.guid))
 
 def list_orphans(server):
     print('Stores without users:')
@@ -196,8 +196,8 @@ def list_orphans(server):
             username = store.user.name if store.user else ''
             storesize = '%.2f MB' % (float(store.size)/2**20)
             storetype = 'public' if store.public else 'private' # XXX archive
-            print(fmt.format(store.guid, _encode(username), '<unknown>', storesize, storetype))
-    print
+            print(fmt.format(store.guid, username, '<unknown>', storesize, storetype))
+    print()
     users = [user for user in server.users() if not user.store]
     list_users('Users without stores', users)
 
@@ -205,15 +205,13 @@ def list_permissions(store):
     if store:
         print('Permissions:')
         for perm in store.permissions():
-            print(_encode('    (store): ' + perm.member.name + ':' + ','.join(perm.rights)))
+            print('    (store): ' + perm.member.name + ':' + ','.join(perm.rights))
         for folder in store.folders():
             for perm in folder.permissions():
-                print(_encode('    ' + folder.path + ': ' + perm.member.name + ':' + ','.join(perm.rights)))
+                print('    ' + folder.path + ': ' + perm.member.name + ':' + ','.join(perm.rights))
 
 def user_counts(server): # XXX allowed/available
-    stats = server.table(PR_EC_STATSTABLE_SYSTEM).dict_(PR_DISPLAY_NAME, PR_EC_STATS_SYSTEM_VALUE)
-    if sys.hexversion >= 0x03000000: # XXX shouldn't be necessary
-        stats = dict([(s.decode('ascii'), stats[s].decode('ascii')) for s in stats])
+    stats = server.stats()
     print('User counts:')
     fmt = '\t{0:>12} {1:>10}'
     print(fmt.format('', 'Used'))
@@ -233,8 +231,8 @@ def quota_str(limit): # TODO in pyko?
 
 def user_details(user):
     fmt = '{0:<30} {1:<}'
-    print(fmt.format('Name:', _encode(user.name)))
-    print(fmt.format('Full name:', _encode(user.fullname)))
+    print(fmt.format('Name:', user.name))
+    print(fmt.format('Full name:', user.fullname))
     print(fmt.format('Email address:', user.email))
     print(fmt.format('Active:', yesno(user.active)))
     print(fmt.format('Administrator:', yesno(user.admin) + (' (system)' if user.admin_level == 2 else '')))
@@ -250,10 +248,10 @@ def user_details(user):
         if user.archive_folder:
             print(fmt.format('Archive folder:', user.archive_folder.path))
 
-        print(fmt.format('Send-as:', ', '.join(_encode(sendas.name) for sendas in user.send_as())))
+        print(fmt.format('Send-as:', ', '.join(sendas.name for sendas in user.send_as())))
         print(fmt.format('Delegation:', '(send only to delegates)' if user.send_only_to_delegates else ''))
         for dlg in user.delegations():
-            print(_encode('    '+dlg.user.name+':'+','.join(dlg.flags)))
+            print('    '+dlg.user.name+':'+','.join(dlg.flags))
         print(fmt.format('Auto-accept meeting requests:', yesno(user.autoaccept.enabled)))
         if user.autoaccept.enabled:
             print(fmt.format('    Accept conflicting:', yesno(user.autoaccept.conflicts)))
@@ -277,31 +275,34 @@ def user_details(user):
 
 def group_details(group):
     fmt = '{0:<16} {1:<}'
-    print(fmt.format('Name:', _encode(group.name)))
+    print(fmt.format('Name:', group.name))
     print(fmt.format('Email address:', group.email))
     print(fmt.format('Address Book:', ('hidden' if group.hidden else 'visible')))
-    print(fmt.format('Send-as:', ', '.join(_encode(sendas.name) for sendas in group.send_as())))
+    print(fmt.format('Send-as:', ', '.join(sendas.name for sendas in group.send_as())))
     list_users('Users', group.users())
 
 def company_details(company, server):
     fmt = '{0:<30} {1:<}'
-    print(fmt.format('Name:', _encode(company.name)))
+    print(fmt.format('Name:', company.name))
     if company.admin:
-        print(fmt.format('Sysadmin:', _encode(company.admin.name)))
+        print(fmt.format('Sysadmin:', company.admin.name))
     print(fmt.format('Address Book:', ('hidden' if company.hidden else 'visible')))
     if company.public_store:
         print(fmt.format('Public store:', company.public_store.guid))
         print(fmt.format('Public store size:', '%.2f MB' % (company.public_store.size / 2**20)))
     if server.multitenant:
-        print(fmt.format('Remote-admin list:', ', '.join(_encode(u.name) for u in company.admins())))
-        print(fmt.format('Remote-view list:', ', '.join(_encode(c.name) for c in company.views())))
+        print(fmt.format('Remote-admin list:', ', '.join(u.name for u in company.admins())))
+        print(fmt.format('Remote-view list:', ', '.join(c.name for c in company.views())))
         user = next(company.users())
-        print(fmt.format('Userquota-recipient list:', ', '.join(_encode(u.name) for u in user.quota.recipients() if u.name != 'SYSTEM')))
-        print(fmt.format('Companyquota-recipient list:', ', '.join(_encode(u.name) for u in company.quota.recipients() if u.name != 'SYSTEM')))
+        print(fmt.format('Userquota-recipient list:', ', '.join(u.name for u in user.quota.recipients() if u.name != 'SYSTEM')))
+        print(fmt.format('Companyquota-recipient list:', ', '.join(u.name for u in company.quota.recipients() if u.name != 'SYSTEM')))
     list_permissions(company.public_store)
 
 def store_details(store, server):
     fmt = '{0:<30} {1:<}'
+    print(fmt.format('Type:', store.type_))
+    if store.user:
+        print(fmt.format('User:', store.user.name))
     print(fmt.format('GUID:', store.guid))
     print(fmt.format('Size:', '%.2f MB' % (store.size / 2**20)))
     list_permissions(store)
@@ -521,6 +522,9 @@ def store_options(name, options, server):
 
     permission_options(store, options, server)
 
+    if options.delete:
+        server.delete(store)
+
 def global_options(options, server):
     if options.lang:
         locale.setlocale(locale.LC_MESSAGES, options.lang)
@@ -538,6 +542,8 @@ def global_options(options, server):
     if options.remove_store:
         server.delete(server.store(options.remove_store))
 
+    if options.list_stores:
+        list_stores(server)
     if options.list_orphans:
         list_orphans(server)
     if options.list_companies:
@@ -588,7 +594,7 @@ def main():
         if args:
             raise Exception("extra argument '%s' specified" % args[0])
 
-        server = kopano.Server(options)
+        server = kopano.server(options, parse_args=True)
         check_options(parser, options, server)
 
         global_options(options, server)
@@ -601,11 +607,14 @@ def main():
         for s in options.stores:
             store_options(s, options, server)
 
+    except (SystemError, KeyboardInterrupt): # TODO SystemError?
+        print()
+        sys.exit(130)
     except Exception as e:
         if 'options' in locals() and options.debug:
             print(traceback.format_exc(), file=sys.stderr)
         else:
-            print(_encode(str(e)), file=sys.stderr)
+            print(str(e), file=sys.stderr)
         sys.exit(1)
 
 if __name__ == '__main__':
