@@ -35,20 +35,6 @@ from kopano.pidlid import (
 
 from . import pst
 
-if sys.hexversion >= 0x03000000:
-    def _encode(s):
-        return s
-
-    def bval(x):
-        return x
-
-else: # pragma: no cover
-    def _encode(s):
-        return s.encode(sys.stdout.encoding or 'utf8')
-
-    def bval(x):
-        return ord(x)
-
 WRAPPED_ENTRYID_PREFIX = codecs.decode('00000000C091ADD3519DCF11A4A900AA0047FAA4', 'hex')
 WRAPPED_EID_TYPE_MASK = 0xf
 WRAPPED_EID_TYPE_CONTACT = 3
@@ -287,8 +273,8 @@ class Service(kopano.Service):
         mapiobj.ModifyRecipients(0, recipients)
 
     def import_pst(self, p, store):
-        folders = p.folder_generator()
-        root_path = rev_cp1252(next(folders).path) # skip root
+        folders = list(p.folder_generator())
+        root_path = rev_cp1252(folders[0].path)
         self.distlist_entryids = []
         self.entryid_map = {}
 
@@ -304,10 +290,14 @@ class Service(kopano.Service):
                     if not import_nids:
                         continue
 
-                path = rev_cp1252(folder.path[len(root_path)+1:])
+                path = rev_cp1252(folder.path[len(root_path)+1:]) or '(root)'
+                if path == '(root)' and folder.ContentCount == 0:
+                    continue
+
                 if self.options.folders and \
                    path.lower() not in [f.lower() for f in self.options.folders]:
                     continue
+
                 self.log.info("importing folder '%s'", path)
                 if self.options.import_root:
                     path = self.options.import_root + '/' + path
@@ -360,7 +350,7 @@ class Service(kopano.Service):
                 if members:
                     for i, (member, oneoff) in enumerate(zip(members, oneoffs)):
                         pos = len(WRAPPED_ENTRYID_PREFIX)
-                        prefix, flags, rest = member[:pos], bval(member[pos]), member[pos+1:]
+                        prefix, flags, rest = member[:pos], member[pos], member[pos+1:]
                         if (prefix == WRAPPED_ENTRYID_PREFIX and \
                             (flags & WRAPPED_EID_TYPE_MASK) in (WRAPPED_EID_TYPE_CONTACT, WRAPPED_EID_TYPE_PERSONAL_DISTLIST) and \
                             rest in self.entryid_map):
@@ -426,8 +416,8 @@ def show_contents(args, options):
     writer = csv.writer(sys.stdout)
     for arg in args:
         p = pst.PST(arg)
-        folders = p.folder_generator()
-        root_path = rev_cp1252(next(folders).path) # skip root
+        folders = list(p.folder_generator())
+        root_path = rev_cp1252(folders[0]).path
 
         if options.summary:
             print("Folders:       %s" % sum(1 for f in folders))
@@ -435,14 +425,16 @@ def show_contents(args, options):
             print("Attachments:   %s" %  p.get_total_attachment_count())
         else:
             for folder in folders:
-                path = rev_cp1252(folder.path[len(root_path)+1:])
+                path = rev_cp1252(folder.path[len(root_path)+1:]) or '(root)'
+                if path == '(root)' and folder.ContentCount == 0:
+                    continue
                 if options.folders and path.lower() not in [f.lower() for f in options.folders]:
                     continue
                 if options.stats:
-                    writer.writerow([_encode(path), folder.ContentCount])
+                    writer.writerow([path, folder.ContentCount])
                 elif options.index:
                     for message in p.message_generator(folder):
-                        writer.writerow([_encode(path), _encode(rev_cp1252(message.Subject or ''))])
+                        writer.writerow([path, rev_cp1252(message.Subject or '')])
 
 
 def create_mapping(args, options):
@@ -495,7 +487,7 @@ def create_mapping(args, options):
 
 
 def main():
-    parser = kopano.parser('cflskpUPuS', usage='kopano-migration-pst PATH [-u NAME]')
+    parser = kopano.parser('CflSKQUPus', usage='kopano-migration-pst PATH [-u NAME]')
     parser.add_option('', '--stats', dest='stats', action='store_true', help='list folders for PATH')
     parser.add_option('', '--index', dest='index', action='store_true', help='list items for PATH')
     parser.add_option('', '--import-root', dest='import_root', action='store', help='import under specific folder', metavar='PATH')
