@@ -2,19 +2,15 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
 from datetime import datetime, timedelta
 import sys
+
 import kopano
 
-if sys.hexversion >= 0x03000000:
-    def _decode(s):
-        return s
-else: # pragma: no cover
-    def _decode(s):
-        return s.decode(getattr(sys.stdin, 'encoding', 'utf8') or 'utf8')
-
 def main():
-    username, config, entryid = [_decode(arg) for arg in sys.argv[1:]]
+    username, config_file, entryid = sys.argv[1:]
+
+    config = kopano.Config(filename=config_file)
+    server = kopano.server(config=config)
 
-    server = kopano.Server()
     user = server.user(username)
     item = user.item(entryid)
     mr = item.meetingrequest
